@@ -5,12 +5,12 @@ require 5.006;
 use strict;
 use warnings;
 
-our $VERSION = '3.58';
+our $VERSION = '3.64';
 $VERSION = eval $VERSION;
 
-use Object::InsideOut::Exception 3.58;
-use Object::InsideOut::Util 3.58 qw(create_object hash_re is_it make_shared);
-use Object::InsideOut::Metadata 3.58;
+use Object::InsideOut::Exception 3.64;
+use Object::InsideOut::Util 3.64 qw(create_object hash_re is_it make_shared);
+use Object::InsideOut::Metadata 3.64;
 
 require B;
 
@@ -97,8 +97,8 @@ if (! exists($GBL{'GBL_SET'})) {
 
         share => {              # Object sharing between threads
             cl  => {},
-            obj => make_shared({}),
             ok  => $threads::shared::threads_shared,
+            # obj               # Tracks TIDs for shared objects
         },
 
         # cache                 # Object initialization activity cache
@@ -790,8 +790,8 @@ sub initialize :Sub(Private)
         }
     }
 
-    # If needed, process any thread object sharing flags
     if ($GBL{'share'}{'ok'}) {
+        # If needed, process any thread object sharing flags
         my $sh_cl = $GBL{'share'}{'cl'};
         foreach my $flag_class (keys(%{$sh_cl})) {
             # Find the class in any class tree
@@ -824,6 +824,13 @@ sub initialize :Sub(Private)
                 }
             }
         }
+
+        # Set up for shared object tracking
+        if (! exists($GBL{'share'}{'obj'}) &&
+            (($] < 5.008009) || ($threads::shared::VERSION lt '1.15')))
+        {
+            $GBL{'share'}{'obj'} = make_shared({});
+        }
     }
 
     # Process field attributes
@@ -1166,9 +1173,9 @@ sub CLONE
     }
 
     # Process thread-shared objects
-    {
+    if (exists($GBL{'share'}{'obj'})) {
         my $sh_obj = $GBL{'share'}{'obj'};
-        lock($sh_obj) if $GBL{'share'}{'ok'};
+        lock($sh_obj);
 
         # Add thread ID to every object in the thread tracking registry
         foreach my $class (keys(%{$sh_obj})) {
@@ -1279,12 +1286,14 @@ sub _obj :Sub(Private)
         threads::shared::share($self);
 
         # Add thread tracking list for this thread-shared object
-        my $sh_obj = $GBL{'share'}{'obj'};
-        lock($sh_obj) if $GBL{'share'}{'ok'};
-        if (exists($$sh_obj{$class})) {
-            $$sh_obj{$class}{$$self} = make_shared([ $GBL{'tid'} ]);
-        } else {
-            $$sh_obj{$class} = make_shared({ $$self => [ $GBL{'tid'} ] });
+        if (exists($GBL{'share'}{'obj'})) {
+            my $sh_obj = $GBL{'share'}{'obj'};
+            lock($sh_obj);
+            if (exists($$sh_obj{$class})) {
+                $$sh_obj{$class}{$$self} = make_shared([ $GBL{'tid'} ]);
+            } else {
+                $$sh_obj{$class} = make_shared({ $$self => [ $GBL{'tid'} ] });
+            }
         }
 
     } elsif ($threads::threads) {
@@ -1814,7 +1823,11 @@ sub DESTROY
     # Workaround for Perl's "in cleanup" bug
     if ($threads::shared::threads_shared && ! $GBL{'term'}) {
         eval {
-            my $bug = keys(%{$GBL{'id'}{'obj'}}) + keys(%{$GBL{'id'}{'reuse'}}) + keys(%{$GBL{'share'}{'obj'}});
+            my $bug = keys(%{$GBL{'id'}{'obj'}})
+                    + keys(%{$GBL{'id'}{'reuse'}})
+                    + ((exists($GBL{'share'}{'obj'}))
+                        ? keys(%{$GBL{'share'}{'obj'}})
+                        : 0);
         };
         if ($@) {
             $GBL{'term'} = 1;
@@ -1830,7 +1843,7 @@ sub DESTROY
             if ($GBL{'term'}) {
                 return if ($tid);   # Continue only if main thread
 
-            } else {
+            } elsif (exists($GBL{'share'}{'obj'})) {
                 my $so_cl = $GBL{'share'}{'obj'}{$class};
                 if (! exists($$so_cl{$$self})) {
                     warn("ERROR: Attempt to DESTROY object ID $$self of class $class in thread ID $tid twice\n");
@@ -1838,8 +1851,15 @@ sub DESTROY
                 }
 
                 # Remove thread ID from this object's thread tracking list
+                # NOTE:  The threads->object() test was added for the case
+                # where OIO objects are passed via Thead::Queue.  I don't
+                # know if this will cause problems with detached threads as
+                # threads->object() returns undef for them.  Also, the main
+                # thread (0) is always a valid thread.
                 lock($so_cl);
-                if (@{$$so_cl{$$self}} = grep { $_ != $tid } @{$$so_cl{$$self}}) {
+                if (@{$$so_cl{$$self}} = grep { ($_ != $tid) &&
+                                                (($_ == 0) || threads->object($_)) }
+                                            @{$$so_cl{$$self}}) {
                     return;
                 }
 
